@@ -1,4 +1,3 @@
-from __future__ import print_function
 from os.path import exists,isfile
 from os import getcwd,chdir,mkdir,listdir,popen #,system,
 from ase.io import read,write
@@ -17,7 +16,7 @@ import tensorflow as tf
 from .md.irmd import IRMD
 from .training import train
 from .training import train_mpnn
-from .AtomDance import AtomDance
+from .AtomDance import AtomDance,check_zmat
 from .dingtalk import send_msg
 import numpy as np
 import json as js
@@ -63,6 +62,7 @@ class LearningMachine(object):
                batch=50,
                step=20000,
                md_step=20,
+               mom_step=100,
                MaxMDstep=500,
                mpopt=1,
                messages=1,
@@ -78,14 +78,16 @@ class LearningMachine(object):
                accMax=0.96,
                accInc=1.0,
                resetAcc=5,
-               learnWay=3,
+               learnWay=5,
                FreeAtoms=None,
+               FirstAtom=None,
                colFrame=10,
                mdInc=1.2,
                rodic=None,
-               rtole=0.55,
-               bondTole=1.3,
-               CheckBond=True,
+               rmin=0.6,
+               rmax=1.3,
+               angmax=15.0,
+               UseZmat=True,
                EngTole=0.05,
                dEtole=0.2,
                d2Etole=0.05,
@@ -117,9 +119,10 @@ class LearningMachine(object):
       self.accMax       = accMax 
       self.accInc       = accInc      # accuracy inrease factor
       self.resetAcc     = resetAcc    # reset accuracy criteria after this train cycle
-      self.rtole        = rtole       # minimum bond length used to supervise ML-MD simulation
-      self.bondTole     = bondTole
-      self.CheckBond    = CheckBond
+      self.rmin         = rmin       # minimum bond length used to supervise ML-MD simulation
+      self.rmax         = rmax
+      self.angmax       = angmax
+      self.UseZmat    = UseZmat
       self.convergence  = convergence
       self.nconvergence = nconvergence
       self.learnWay     = learnWay    # the way of learing
@@ -138,6 +141,7 @@ class LearningMachine(object):
       self.T            = T
       self.Tmax         = Tmax
       self.md_step      = md_step
+      self.mom_step     = mom_step
       self.colFrame     = colFrame
       self.EngTole      = EngTole
       self.dEtole       = dEtole
@@ -160,10 +164,12 @@ class LearningMachine(object):
       self.EnergyFunction = EnergyFunction
       self.MessageFunction= MessageFunction
       self.freeatoms    = FreeAtoms
-      self.c            = AtomDance(poscar=self.initConfig,nn=self.nn,
-                                    rtole=self.rtole-0.05,bondTole=self.bondTole)
-      self.natom        = self.c.natom
-      self.atom_name    = self.c.atom_name
+      self.a            = AtomDance(poscar=self.initConfig,nn=self.nn,
+                                    rmin=self.rmin-0.05,rmax=self.rmax,
+                                    FirstAtom=FirstAtom,freeatoms=self.freeatoms)
+      self.a.get_freebond(freeatoms=self.freeatoms)
+      self.natom        = self.a.natom
+      self.atom_name    = self.a.atom_name
       self.get_ro(rodic)
       self.get_fixatom()
 
@@ -182,7 +188,7 @@ class LearningMachine(object):
          if len(aimd_)>1:
             self.label  = aimd_[-1]
          else:
-            self.label  = self.c.label
+            self.label  = self.a.label
       else:
          self.label     = label
 
@@ -200,7 +206,6 @@ class LearningMachine(object):
            print('------------------------------------------------------------------------',file=l)
            print('\n',file=l)
 
-
   def run(self):
       ''' recursive training loop '''
       it       = []
@@ -214,56 +219,83 @@ class LearningMachine(object):
       if not exists(self.aidir):
          mkdir(self.aidir)
 
-      i            = 0
+      iter_        = 0
       step         = 0
       mdsteps      = self.md_step
+      momsteps     = self.mom_step
       data_dir     = {}
       running      = True
-      bondBroken   = 0
+      Deformed     = 0
+      if self.learnWay==5:
+         if self.freeatoms is None:
+            self.freeatoms = []
+         ms        = len(self.freeatoms)
+      else:
+         ms        = 0
       self.save_config()
       converg      = False
       nconvergence = 0
-      lw4          = True
+      unreasonable_zv,uzv_ = None,None
+      zmat_variable= None
+      zmatopt      = None
+      zmats        = None
+      zvlo,zvhi    = 0.0,0.0
+      u_zvlo,u_zvhi= 0.0,0.0
 
       while running:
+          optlog   = None
+          relaxlog = None
           learnWay = self.learnWay
-          run_dir  = self.aidir+'/'+self.label+'-'+str(i)
-          data_dir[self.label+'-'+str(i)] = cwd+'/'+run_dir+'/'+self.label+'.traj'
+          mom_step = self.mom_step*(ms+1) if learnWay==5 else self.mom_step
+          run_dir  = self.aidir+'/'+self.label+'-'+str(iter_)
+          data_dir[self.label+'-'+str(iter_)] = cwd+'/'+run_dir+'/'+self.label+'.traj'
 
           if exists(run_dir):
-             i += 1
+             iter_ += 1
              continue
-
           if isfile('md.traj'): 
              atoms = read('md.traj',index=-1)
-             atoms.set_constraint(self.fixatoms)
+             # print('-  atomic structure from MD trajectories.')
           else:
+             print('-  cannot find MD trajectory, use learnWay=1 in the first iter.')
              atoms = read(self.initConfig)
-             atoms = self.c.bond_momenta_bigest(atoms)
-             atoms.set_constraint(self.fixatoms)
-             e_gmd,mdsteps = self.mlmd(cwd,run_dir,atoms,10000,i,learnWay) 
- 
-          if learnWay==4 and lw4:
-             atoms,lw4 = self.c.bond_momenta(atoms)
-          else:
-             bondBroken,bkbd = self.c.check_bond(atoms=atoms,bondTole=self.bondTole)
-          if not lw4: learnWay = 3   
-
-          if bondBroken>=1 and self.CheckBond and (not bkbd is None):  # if bonds have broken
-             mdsteps = 10
-             images = self.c.stretch(bkbd,atoms=atoms,nbin=mdsteps,st=self.bondTole-0.015,ed=1.0,scale=1.2,traj='md.traj')
-             if images is None:
-                atoms = self.c.bond_momenta_bigest(atoms)
-                e_gmd,mdsteps = self.mlmd(cwd,run_dir,atoms,10000,i,learnWay) 
-             else:
-                atoms  = images[-1]
-                bondBroken,bkbd = self.c.check_bond(atoms=atoms,bondTole=self.bondTole)
-
+             learnWay = 1
+          
+          if learnWay==4 or learnWay==5:
+             momsteps += mdsteps
+             if momsteps >= mom_step:
+                momsteps  = 0
+                if learnWay == 4:
+                   atoms     = self.a.bond_momenta(atoms)
+                elif learnWay == 5:
+                   if ms<self.natom:
+                      i = self.a.zmat_id[ms]
+                      j = self.a.zmat_id[ms+1]
+                      if i not in self.freeatoms: 
+                         self.freeatoms.append(i)
+                         for i_ in self.a.neighbors[i]:
+                             if len(self.a.neighbors[i_])==1:
+                                if i_ not in self.freeatoms: 
+                                   self.freeatoms.append(i_)
+                      if j not in self.freeatoms: 
+                         self.freeatoms.append(j)
+                         for j_ in self.a.neighbors[j]:
+                             if len(self.a.neighbors[j_])==1:
+                                if j_ not in self.freeatoms: 
+                                   self.freeatoms.append(j_)
+                      self.save_config()
+                      atoms = self.a.set_bond_momenta(i,j,atoms)
+                   learnWay = 4
+                learnWay  = 2
+                ms       += 1
+          self.get_fixatom()
+          atoms.set_constraint(self.fixatoms)
           atoms.write('poscar.gen')                                    # for aimd
-          if learnWay>=3: 
-             dft_step = int(mdsteps/self.colFrame)+1
-          else:
+
+          if learnWay==1 : 
              dft_step = self.dft_step
+          else:
+             dft_step = int(mdsteps/self.colFrame)+1
 
           e_aimd,eml_,dEmax_,d2Emax_ = self.aimd(cwd,run_dir,dft_step,gen,learnWay)
           e_siesta.append(e_aimd[0])                                   # get siesta results              
@@ -280,7 +312,7 @@ class LearningMachine(object):
           accu = -1000.0                                               # acc >= accMin
           tIter= 1   
           loss = self.lossCriteria+0.1      
-          training = True                      
+          training = True                   
           while (accu<=self.accCriteria or loss>=self.lossCriteria) and training:        
                 self.load_config()
                 loss,accu,accMax,p,zpe,tstep = self.trainer(direcs=trajs_,
@@ -308,9 +340,31 @@ class LearningMachine(object):
                 tIter += 1
 
           self.lossCriteria = float(loss)
+          if self.UseZmat:
+             if not zmatopt is None:
+                optlog = self.a.get_optimal_zv(atoms,zmatopt,self.initConfig)
+             unreasonable_zv,u_zvlo,u_zvhi = self.a.get_zmat_info(zmats)
+
+          if mdsteps<10: mdsteps = 10
+          zmats   = None
+          images  = None
+          zmatopt = zmat_variable
+          if Deformed>=1.0:
+             images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=zmat_variable,nbin=mdsteps,
+                                                 zvlo=zvlo,zvhi=zvhi,traj='md.traj')
+          elif not unreasonable_zv is None:
+             images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=unreasonable_zv,nbin=mdsteps,
+                                                 zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj')
+          else:
+             e_gmd,mdsteps,Deformed,zmats,zmat_variable,zvlo,zvhi = self.mlmd(cwd,run_dir,atoms, 10000,iter_,learnWay) 
+             if self.UseZmat and mdsteps<=2:
+                mdsteps= 7
+                images,relaxlog = self.a.zmat_relax(atoms=atoms,nbin=20,relax_step=mdsteps,traj='md.traj')
+          if not images is None:
+             Deformed,zmat,zmat_variable,zvlo,zvhi = check_zmat(atoms=images[-1],rmin=self.rmin,rmax=self.rmax,
+                                    angmax=self.angmax,zmat_id=self.a.zmat_id,
+                                    zmat_index=self.a.zmat_index,InitZmat=self.a.InitZmat)
 
-          if bondBroken<1 or (not self.CheckBond): 
-             e_gmd,mdsteps = self.mlmd(cwd,run_dir,atoms,10000,i,learnWay) 
           step += mdsteps                                          # run ML-MD test training results     
 
           if accu >= self.accCriteria and tstep<self.step and tIter<=2: # get md steps
@@ -328,22 +382,14 @@ class LearningMachine(object):
 
           self.save_config()
 
-          if learnWay>=3: e_gmd = eml_
+          if learnWay==3 or learnWay==4 or learnWay==5: e_gmd = eml_
           e_gulp.append(e_gmd[0])
           diff  = abs(e_gmd[0]-e_aimd[0])
-          it.append(i)
-
-          plot_energies(it,e_siesta,e_gulp)                             # plot learning status 
-                                                                        #
-         #  with open('energies.log','a') as fe:                        # write learning status 
-         #       for i_,e_ in enumerate(e_gmd):                         # to file
-         #           if i_==0:                                          #
-         #              fe.write(' %d %f %f diff: %f \n' %(step+i_,e_gmd[i_],e_aimd[0],diff))
-         #           else:                                              #
-         #              fe.write(' %d %f \n' %(step+i_,e_gmd[i_]))      #
-          self.learninglog(i,loss,accu,mdsteps,e_gulp[-1],e_siesta[-1],dEmax_,d2Emax_,learnWay,
-                           bondBroken,bkbd)
+          it.append(iter_)
 
+          plot_energies(it,e_siesta,e_gulp)                             # plot learning status                                   
+          self.learninglog(iter_,loss,accu,mdsteps,e_gulp[-1],e_siesta[-1],dEmax_,d2Emax_,learnWay,Deformed,
+                           self.a.InitZmat,unreasonable_zv,u_zvlo,u_zvhi,optlog,relaxlog)
           converg_ = converg
           if diff<=self.convergence: 
              converg = True
@@ -359,14 +405,14 @@ class LearningMachine(object):
              send_msg('-  Convergence occured.')
              return diff
 
-          if i>self.maxiter:
+          if iter_>self.maxiter:
              print('-  Max iteration reached,the loss %7.4f and accuracy %7.4f.' %(loss,accu))
              send_msg('-  Max iteration reached, the loss %7.4f and accuracy %7.4f.' %(loss,accu))
              return diff
-          i += 1
+          iter_ += 1
 
-
-  def learninglog(self,i,loss,accu,mdsteps,eirff,edft,dEmax,d2Emax,learnWay,bondBroken,bkbd):
+  def learninglog(self,i,loss,accu,mdsteps,eirff,edft,dEmax,d2Emax,learnWay,Deformed,
+                  InitZmat,uz,uzlo,uzhi,optlog,relaxlog):
       with open('learning.log','a') as l:
          l.write('                  ====================================                  \n')
          l.write('                      Learning iteration:  %4d                         \n' %i)
@@ -378,12 +424,16 @@ class LearningMachine(object):
          l.write('Energy from ML and DFT:  %f  .vs.  %f \n' %(eirff,edft))
          l.write('The max first derivative and second derivative:  %f,  %f \n' %(dEmax,d2Emax))
          l.write('The learning method of this iter:  %d \n' %learnWay)
-         # l.write('Bond Broken :  %d \n' %bondBroken)
-         if bondBroken>=1 and (not bkbd is None):
-            l.write('Bond between atoms %d and %d was broken during MLMD.\n' %(bkbd[0],bkbd[1]))
+         if not relaxlog is None:
+            if Deformed>=1:
+               l.write('Molecule structure is deformed, using zmatmix relax\n' )
+            elif not uz is None:
+               l.write('relax unreasonable zmatrix variable:\n')
+            l.write(relaxlog)
+         if not optlog is None:
+            l.write(optlog)
          l.write(' \n')
 
-
   def aimd(self,cwd,run_dir,tstep,gen,learnWay):
       mkdir(run_dir)
       chdir(cwd+'/'+run_dir)
@@ -392,12 +442,13 @@ class LearningMachine(object):
       dEmax = 0.0
       d2Emax = 0.0
 
-      if learnWay<=2:
+      if learnWay==1:
          images = siesta_md(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,us='F',tstep=tstep,
                             gen=cwd+'/'+gen,FreeAtoms=self.freeatoms,
                             xcf=self.xcf,xca=self.xca,basistype=self.basistype)
          eaimd = [images[0].get_potential_energy()]
-      elif learnWay>=3:
+         popen('cp %s ../../md.traj' %(self.label+'.traj'))
+      elif learnWay>=2:
          popen('cp ../../md.traj ./')
          popen('cp ../../ffield.json ./')
          # print(' * files in this dir \n',listdir())
@@ -408,26 +459,29 @@ class LearningMachine(object):
                                                  cpu=self.ncpu)
          eaimd = E
          emlmd = E_
+      else:
+         raise RuntimeError('-  learn way not supported yet!')
       chdir(cwd)
       return eaimd,emlmd,dEmax,d2Emax
 
-
   def mlmd(self,cwd,run_dir,atoms,Tmax,Iter,learnWay):        # run classic MD to test training results
+      mdstep = max(int(self.md_step/5),2) if learnWay==2 else self.md_step
       irmd = IRMD(atoms=atoms,label=self.label,
-                  time_step=self.dt_mlmd,totstep=self.md_step,Tmax=Tmax,
-                  ro=self.ro,rtole=self.rtole,Iter=Iter,initT=self.T,
-                  bondTole=self.bondTole,
-                  CheckDE=True,dEstop=self.dEstop,
+                  time_step=self.dt_mlmd,totstep=mdstep,Tmax=Tmax,
+                  Iter=Iter,initT=self.T,
+                  ro=self.ro,rmin=self.rmin,rmax=self.rmax,angmax=self.angmax,
+                  UseZmat=self.UseZmat,InitZmat=self.a.InitZmat,
+                  zmat_id=self.a.zmat_id,zmat_index=self.a.zmat_index,
+                  dEstop=self.dEstop,
                   nn=self.nn,vdwnn=self.vdwnn)
       if learnWay==2:
-         irmd.opt()
+         Deformed,zmats,zv,zvlo,zvhi = irmd.opt()
       else:
-         irmd.run()
+         Deformed,zmats,zv,zvlo,zvhi = irmd.run()
       mdsteps= irmd.step
-      Emd  = irmd.Epot
+      Emd    = irmd.Epot
       irmd.close()
-      return Emd,mdsteps
-
+      return Emd,mdsteps,Deformed,zmats,zv,zvlo,zvhi
 
   def get_ro(self,rodic):
       if rodic is None:
@@ -437,7 +491,7 @@ class LearningMachine(object):
                       'H-H':0.8,
                       'others':1.35} 
       else:
-      	 self.rodic= rodic
+           self.rodic= rodic
 
       self.natom     = len(self.atom_name)
       self.ro   = np.zeros([self.natom,self.natom])
@@ -453,13 +507,12 @@ class LearningMachine(object):
               else:
                  self.ro[i][j] = self.rodic['others']
 
-
   def get_fixatom(self):
-      atoms             = self.c.ir.atoms
+      atoms             = self.a.ir.atoms
       if self.freeatoms is None:
          self.fixatoms  = None
       else:
-         self.fixatoms  = FixAtoms(indices=[atom.index for atom in atoms if atom.index not in FreeAtoms])
+         self.fixatoms  = FixAtoms(indices=[atom.index for atom in atoms if atom.index not in self.freeatoms])
 
 
   def save_config(self):
@@ -467,7 +520,8 @@ class LearningMachine(object):
            InPut = {'accCriteria':self.accCriteria,
                     'lossCriteria':self.lossCriteria,
                     'lossTole':self.lossTole,
-                    'bondTole':self.bondTole,
+                    'rmax':self.rmax,
+                    'angmax':self.angmax,
                     'accTole':self.accTole,
                     'accMax':self.accMax,
                     'accInc':self.accInc,
@@ -475,6 +529,7 @@ class LearningMachine(object):
                     'step':self.step,
                     'dft_step':self.dft_step,
                     'md_step':self.md_step,
+                    'mom_step':self.mom_step,
                     'MaxMDstep':self.MaxMDstep,
                     'EngTole':self.EngTole,
                     'dEtole':self.dEtole,
@@ -483,7 +538,7 @@ class LearningMachine(object):
                     'mdInc':self.mdInc,
                     'dt_aimd':self.dt_aimd,
                     'dt_mlmd':self.dt_mlmd,
-                    'rtole':self.rtole,
+                    'rmin':self.rmin,
                     'Tmax':self.Tmax,
                     'T':self.T,
                     'maxiter':self.maxiter,
@@ -492,18 +547,19 @@ class LearningMachine(object):
                     'mefac':self.mefac,
                     'learnWay':self.learnWay,
                     'colFrame':self.colFrame,
+                    'UseZmat':self.UseZmat,
                     'freeatoms':self.freeatoms,
                     'bore':self.bore}
            js.dump(InPut,fj,sort_keys=True,indent=2)
 
-
   def load_config(self):
       with open('options.json','r') as fj:
            InPut = js.load(fj)
            self.accCriteria  = InPut['accCriteria']
            self.lossCriteria = InPut['lossCriteria']
            self.lossTole     = InPut['lossTole']
-           self.bondTole     = InPut['bondTole']
+           self.rmax         = InPut['rmax']
+           self.angmax      = InPut['angmax']
            self.accTole      = InPut['accTole']
            self.accMax       = InPut['accMax']
            self.accInc       = InPut['accInc']
@@ -511,6 +567,7 @@ class LearningMachine(object):
            self.step         = InPut['step'] 
            self.dft_step     = InPut['dft_step'] 
            self.md_step      = InPut['md_step'] 
+           self.mom_step     = InPut['mom_step'] 
            self.MaxMDstep    = InPut['MaxMDstep'] 
            self.EngTole      = InPut['EngTole'] 
            self.dEtole       = InPut['dEtole'] 
@@ -519,7 +576,7 @@ class LearningMachine(object):
            self.mdInc        = InPut['mdInc'] 
            self.dt_aimd      = InPut['dt_aimd'] 
            self.dt_mlmd      = InPut['dt_mlmd'] 
-           self.rtole        = InPut['rtole'] 
+           self.rmin        = InPut['rmin'] 
            self.Tmax         = InPut['Tmax'] 
            self.T            = InPut['T'] 
            self.maxiter      = InPut['maxiter']
@@ -528,17 +585,18 @@ class LearningMachine(object):
            self.mefac        = InPut['mefac']
            self.learnWay     = InPut['learnWay']
            self.colFrame     = InPut['colFrame']
+           self.UseZmat    = InPut['UseZmat']
            self.bore         = InPut['bore']
-           self.freeatoms    = InPut['freeatoms']
-
-
+           if self.freeatoms!= InPut['freeatoms']:
+              self.freeatoms = InPut['freeatoms']
+              self.a.get_freebond(self.freeatoms)
+           
   def close(self):
       print('-  LM compeleted.')
       self.atom_name = None
       self.ro        = None
 
 
-
 if __name__ == '__main__':
    direcs = {'nm6_5':'/home/feng/siesta/nm6_5',
              'nm6_14':'/home/feng/siesta/nm6_14' }
@@ -568,7 +626,7 @@ if __name__ == '__main__':
                        messages=1,
                        learnWay=3,
                        colFrame=10,
-                       rtole=0.6,
+                       rmin=0.6,
                        EngTole=0.1,
                        dEtole=0.20,
                        d2Etole=0.05,
