@@ -47,10 +47,15 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
         energies.append(energy)
         dEs.append(dE_)
         d2Es.append(d2E_)
+        
         if select:
            if dE_>dE or d2E_>d2E:
               if i not in ind_:
-                 ind_.append(i)
+                 if len(ind_)>0:
+                    if i-ind_[-1]>1:
+                       ind_.append(i)
+                 else:
+                    ind_.append(i)
 
     ide  = np.argmax(dEs)
     id2e = np.argmax(d2Es)
