@@ -77,15 +77,15 @@ def freq(atoms=None):
 
 
 def md(atoms=None,gen='poscar.gen',step=100,model='mpnn',initT=300,timeStep=0.1,
-       vdwnn=False,v=True):
+       vdwnn=False):
     irmd = IRMD(time_step=timeStep,totstep=step,atoms=atoms,gen=gen,Tmax=10000,
-                ro=0.8,rtole=0.5,initT=initT,vdwnn=vdwnn)
+                ro=0.8,rmin=0.5,initT=initT,vdwnn=vdwnn)
     irmd.run()
 
     irmd.close()
     del irmd
     images = Trajectory('md.traj')
-    if v:
-       view(images[-1]) # ,viewer='x3d'
+   #  if v:
+   #     view(images[-1]) # ,viewer='x3d'
     return images[-1]
 
