@@ -3,6 +3,7 @@
 import argh
 import argparse
 from ..irff import IRFF
+from ..AtomDance import get_zmat_variable,get_zmat_angle,get_zmatrix,check_zmat
 from ase.io import read,write
 import numpy as np
 from ase.optimize import BFGS
@@ -37,9 +38,10 @@ class IRMD(object):
   ''' Intelligent Reactive Molecular Dynamics '''
   def __init__(self,label=None,atoms=None,gen='poscar.gen',ffield='ffield.json',
                index=-1,totstep=100,vdwnn=False,nn=True,
-               initT=300,Tmax=10000,time_step=0.1,
-               ro=None,rtole=0.6,Iter=0,bondTole=1.3,
-               CheckDE=False,dEstop=0.5):
+               initT=300,Tmax=10000,time_step=0.1,Iter=0,
+               ro=None,rmin=0.6,rmax=1.3,angmax=20.0,
+               UseZmat=False,zmat_id=None,zmat_index=None,InitZmat=None,
+               dEstop=0.5):
       self.Epot    = []
       self.epot    = 0.0
       self.ekin    = 0.0
@@ -48,25 +50,30 @@ class IRMD(object):
       self.Tmax    = Tmax
       self.totstep = totstep
       self.ro      = ro
-      self.rtole   = rtole
+      self.rmin    = rmin
       self.Iter    = Iter
       self.atoms   = atoms
       self.time_step = time_step
       self.step    = 0
-      self.bondTole= bondTole
-      self.CheckDE   = CheckDE
+      self.rmax    = rmax
+      self.angmax  = angmax
+      self.UseZmat = UseZmat
+      self.zmat_id   = zmat_id
+      self.zmat_index= zmat_index
+      self.InitZmat  = InitZmat
+      self.zmats     = []
       self.dEstop    = dEstop
 
       if self.atoms is None:
-         self.atoms   = read(gen,index=index)
+         self.atoms  = read(gen,index=index)
       
-      self.atoms.calc = IRFF(atoms=self.atoms, mol=label,libfile=ffield,
+      self.atoms.calc= IRFF(atoms=self.atoms, mol=label,libfile=ffield,
                              rcut=None,nn=nn,vdwnn=vdwnn)
       self.natom     = len(self.atoms)
       self.re        = self.atoms.calc.re
       self.dyn       = None
       self.atoms.calc.calculate(atoms=self.atoms)
-      self.InitBonds = getBonds(self.natom,self.atoms.calc.r,self.bondTole*self.re)
+      self.InitBonds = getBonds(self.natom,self.atoms.calc.r,self.rmax*self.re)
 
       if (self.atoms is None) and gen.endswith('.gen'):
          MaxwellBoltzmannDistribution(self.atoms, self.initT*units.kB)
@@ -80,14 +87,44 @@ class IRMD(object):
          else:
             MaxwellBoltzmannDistribution(self.atoms, self.initT*units.kB)
 
-
   def run(self):
+      self.zv,self.zvlo,self.zvhi = None,0.0,0.0
       self.dyn = VelocityVerlet(self.atoms, self.time_step*units.fs,trajectory='md.traj')  
       def printenergy(a=self.atoms):
           epot_      = a.get_potential_energy()
-          r          = a.calc.r.detach().numpy()
-          i_         = np.where(np.logical_and(r<self.rtole*self.ro,r>0.0001))
-          n          = len(i_[0])
+          n,self.Deformed = 0,0
+          if self.UseZmat:
+             self.Deformed,zmat,self.zv,self.zvlo,self.zvhi = check_zmat(atoms=a,rmin=self.rmin,rmax=self.rmax,
+                                             angmax=self.angmax,zmat_id=self.zmat_id,
+                                             zmat_index=self.zmat_index,InitZmat=self.InitZmat)
+             bonds      = getBonds(self.natom,self.atoms.calc.r,1.22*self.re)
+             newbond    = self.checkBond(bonds)
+             if newbond: 
+                self.zv = None
+                self.Deformed += 0.2
+                bonds      = getBonds(self.natom,self.atoms.calc.r,1.25*self.re)
+                newbond    = self.checkBond(bonds)
+                if newbond: 
+                   self.Deformed += 0.2
+                   bonds      = getBonds(self.natom,self.atoms.calc.r,1.23*self.re)
+                   newbond    = self.checkBond(bonds)
+                   if newbond: 
+                      self.Deformed += 0.2
+                      bonds      = getBonds(self.natom,self.atoms.calc.r,1.21*self.re)
+                      newbond    = self.checkBond(bonds)
+                      if newbond: 
+                         self.Deformed += 0.2
+                         bonds      = getBonds(self.natom,self.atoms.calc.r,1.19*self.re)
+                         newbond    = self.checkBond(bonds)
+                         if newbond: 
+                            self.Deformed += 0.2
+                            bonds      = getBonds(self.natom,self.atoms.calc.r,1.17*self.re)
+                            newbond    = self.checkBond(bonds)
+             self.zmats.append(zmat)
+          else:
+             r        = a.calc.r.detach().numpy()
+             i_       = np.where(np.logical_and(r<self.rmin*self.ro,r>0.0001))
+             n        = len(i_[0])
 
           if len(self.Epot)==0:
              dE_ = 0.0
@@ -104,8 +141,8 @@ class IRMD(object):
                 'Etot = %.3feV' % (self.step,self.epot,self.ekin,self.T,
                                    self.epot + self.ekin))
           try:
-             if self.CheckDE:
-                assert n==0 and dE_<self.dEstop,'Atoms too closed or Delta E too high!' 
+             if self.UseZmat:
+                assert self.Deformed<1,'Atoms structure is deformed!' 
              else:
                 assert n==0 and self.T<self.Tmax,'Atoms too closed or Temperature goes too high!' 
           except:
@@ -113,8 +150,8 @@ class IRMD(object):
              #     print('atoms pair',_)
              if n>0: 
                  print('Atoms too closed, stop at %d.' %self.step)
-             elif dE_>=self.dEstop: 
-                 print('dE = %f exceed the limit, stop at %d.' %(dE_,self.step))
+             elif self.Deformed>=1.0: 
+                 print('Structure Deformed = %f exceed the limit, stop at %d.' %(self.Deformed,self.step))
              else:
                  print('unknown reason, stop at %d.' %self.step)
              self.dyn.max_steps = self.dyn.nsteps-1
@@ -123,49 +160,79 @@ class IRMD(object):
       self.dyn.attach(printenergy,interval=1)
       # self.dyn.attach(traj.write,interval=1)
       self.dyn.run(self.totstep)
-      # bonds      = getBonds(self.natom,self.atoms.calc.r,self.bondTole*self.re)
-      # bondBroken = self.checkBond(bonds)
-      # return bondBroken
-
+      return self.Deformed,self.zmats,self.zv,self.zvlo,self.zvhi
 
   def opt(self):
+      self.zv,self.zvlo,self.zvhi = None,0.0,0.0
       self.dyn = BFGS(self.atoms,trajectory='md.traj')
       def check(a=self.atoms):
           epot_      = a.get_potential_energy()
-          r          = a.calc.r.detach().numpy()
-          i_         = np.where(np.logical_and(r<self.rtole*self.ro,r>0.0001))
-          n          = len(i_[0])
+          n,self.Deformed = 0,0
+          if self.UseZmat:
+             Deformed,zmat,self.zv,self.zvlo,self.zvhi = check_zmat(atoms=a,rmin=self.rmin,rmax=self.rmax,angmax=self.angmax,
+                                        zmat_id=self.zmat_id,zmat_index=self.zmat_index,
+                                        InitZmat=self.InitZmat)
+             bonds      = getBonds(self.natom,self.atoms.calc.r,1.18*self.re)
+             newbond    = self.checkBond(bonds)
+             if newbond: 
+                self.zv = None
+                self.Deformed += 0.2
+                bonds      = getBonds(self.natom,self.atoms.calc.r,1.25*self.re)
+                newbond    = self.checkBond(bonds)
+                if newbond: 
+                   self.Deformed += 0.2
+                   bonds      = getBonds(self.natom,self.atoms.calc.r,1.23*self.re)
+                   newbond    = self.checkBond(bonds)
+                   if newbond: 
+                      self.Deformed += 0.2
+                      bonds      = getBonds(self.natom,self.atoms.calc.r,1.21*self.re)
+                      newbond    = self.checkBond(bonds)
+                      if newbond: 
+                         self.Deformed += 0.2
+                         bonds      = getBonds(self.natom,self.atoms.calc.r,1.19*self.re)
+                         newbond    = self.checkBond(bonds)
+                         if newbond: 
+                            self.Deformed += 0.2
+                            bonds      = getBonds(self.natom,self.atoms.calc.r,1.17*self.re)
+                            newbond    = self.checkBond(bonds)
+             self.zmats.append(zmat)
+          else:
+             r          = a.calc.r.detach().numpy()
+             i_         = np.where(np.logical_and(r<self.rmin*self.ro,r>0.0001))
+             n          = len(i_[0])
           
           self.Epot.append(epot_)
           self.epot  = epot_/self.natom
 
           self.step  = self.dyn.nsteps
           try:
-             assert n==0 and (not np.isnan(epot_)),'Atoms too closed!'
+             if self.UseZmat:
+                assert Deformed<1,'Atoms too closed or Deformed!' 
+             else:
+                assert n==0 and (not np.isnan(epot_)),'Atoms too closed!'
           except:
-             for _ in i_:
-                 print('atoms pair',_)
-             print('Atoms too closed or temperature too high, stop at %d.' %self.step)
+             if n>0: 
+                 print('Atoms too closed, stop at %d.' %self.step)
+             elif self.Deformed>=1.0: 
+                 print('Structure Deformed = %f exceed the limit, stop at %d.' %(self.Deformed,self.step))
+             elif np.isnan(epot_):
+                 print('potential energy is NAN, stop at %d.' %self.step)
+             else:
+                 print('unknown reason, stop at %d.' %self.step)
              self.dyn.max_steps = self.dyn.nsteps-1
-             # raise ValueError('-  Energy is NaN!' )
   
       self.dyn.attach(check,interval=1)
       self.dyn.run(0.00001,self.totstep)
-      
-      bonds      = getBonds(self.natom,self.atoms.calc.r,self.bondTole*self.re)
-      bondBroken = self.checkBond(bonds)
-      return bondBroken
-
+      return self.Deformed,self.zmats,self.zv,self.zvlo,self.zvhi
 
   def checkBond(self,bonds):
-      bondBroken = False
-      for bd in self.InitBonds:
+      newbond = False
+      for bd in bonds:
           bd_ = (bd[1],bd[0])
-          if (bd not in bonds) and (bd_ not in bonds):
-             bondBroken = True
-             return bondBroken 
-      return bondBroken
-
+          if (bd not in self.InitBonds) and (bd_ not in self.InitBonds):
+             newbond = True
+             return newbond 
+      return newbond
 
   def logout(self):
       with open('md.log','w') as fmd:
@@ -225,7 +292,6 @@ class IRMD(object):
          fmd.write('\n------------------------------------------------------------------------\n')
          fmd.write('\n- Machine Learning MD Completed!\n')
 
-
   def close(self):
       self.logout()
       self.dyn   = None
