@@ -27,7 +27,7 @@ class MPNN(ReaxFF):
   def __init__(self,libfile='ffield',direcs={},
                dft='ase',atoms=None,
                cons=['val','vale','rosi','ropi','ropp', # 'ovun3', # 'val8','val9','val10',
-                     'bo1','bo2','bo3','bo4','bo5','bo6','Depi','Depp','hbtol'],
+                     'bo1','bo2','bo3','bo4','bo5','bo6','Depi','Depp','lp3'], # 'hbtol'
                optmol=True,mefac=0.1,
                opt=None,optword='nocoul',
                nanv={'boc1':-2.0},
@@ -44,7 +44,6 @@ class MPNN(ReaxFF):
                nn=True,
                messages=1,
                TwoBodyOnly=False,
-               bopSV=False,
                mpopt=None,
                bo_layer=[2,1],
                bf_layer=[6,1],
@@ -83,7 +82,6 @@ class MPNN(ReaxFF):
       self.be_layer         = be_layer
       self.vdw_layer        = vdw_layer
       self.vdwnn            = vdwnn
-      self.bopSV            = bopSV
       if mpopt is None:
          self.mpopt = [True for i in range(messages+3)]
       else:
@@ -110,7 +108,6 @@ class MPNN(ReaxFF):
       self.esi      = {}
       self.fbo      = {}
 
-
   def supervise(self):
       ''' supervised learning term'''
       l_atol = 0.0
@@ -136,29 +133,24 @@ class MPNN(ReaxFF):
              fao_= tf.where(tf.less_equal(self.rbd[bd],bm_*self.re[bd]),1.0,0.0) ##### r< 1.2*re that bo > botol
              diffa_ = tf.reduce_sum(input_tensor=tf.nn.relu(self.botol-self.bo0[bd])*fao_)
              l_atol = tf.add(diffa_,l_atol)
-             
-             if self.bopSV:
-                fbo = tf.where(tf.less_equal(self.rbd[bd],self.rcuta[bd]),1.0,0.0) ##### r> rcut that bo < botol
-                diffb_  = tf.reduce_sum(input_tensor=tf.nn.relu(self.atol - self.bop_si[bd])*fbo)
-                l_atol  = tf.add(diffb_,l_atol)
-                diffb_  = tf.reduce_sum(input_tensor=tf.nn.relu(self.atol - self.bop_pi[bd])*fbo)
-                l_atol  = tf.add(diffb_,l_atol)
-                diffb_  = tf.reduce_sum(input_tensor=tf.nn.relu(self.atol - self.bop_pp[bd])*fbo)
-                l_atol  = tf.add(diffb_,l_atol)
+
+             #  bop should be zero if r>rcut
+             fbo = tf.where(tf.less_equal(self.rbd[bd],self.rc_bo[bd]),0.0,1.0) ##### r> rc_bo that bo < botol
+             diffb_  = tf.reduce_sum(self.bop[bd]*fbo)
+             l_atol  = tf.add(diffb_,l_atol)
 
              fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< re that bo > bore_
              self.diffe[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.bo0[bd])*fe))
-             l_atol = tf.add(self.diffe[bd],l_atol)
-
-             l_atol = tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
+             l_atol  = tf.add(self.diffe[bd],l_atol)
+             l_atol= tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
       return l_atol*self.bo_penalty
 
-
   def get_loss(self):
       self.Loss = 0.0
       for mol in self.mols:
-          if mol in self.weight:
-             w_ = self.weight[mol]
+          mol_ = mol.split('-')[0]
+          if mol_ in self.weight:
+             w_ = self.weight[mol_]
           else:
              w_ = self.weight['others']
 
@@ -190,7 +182,6 @@ class MPNN(ReaxFF):
          self.Loss  += self.ME*self.mefac
       self.accuracy  = self.accuracy/self.nmol
 
-
   def build_graph(self):
       print('-  building graph ...')
       self.accuracy   = tf.constant(0.0,name='accuracy')
@@ -210,7 +201,6 @@ class MPNN(ReaxFF):
       self.get_loss()
       print('-  end of build.')
 
-
   def get_atomic_energy(self):
       i = 0
       for sp in self.spec:
@@ -225,7 +215,6 @@ class MPNN(ReaxFF):
           zpe_ = tf.gather_nd(self.EATOM,self.atomlink[mol]) 
           self.zpe[mol] = tf.reduce_sum(input_tensor=zpe_,name='zpe') + self.MolEnergy[mols]
 
-
   def get_total_energy(self):
       for mol in self.mols:
           # mols = mol.split('-')[0] 
@@ -252,7 +241,6 @@ class MPNN(ReaxFF):
                                   self.eself[mol], 
                                   self.zpe[mol],name='E_%s' %mol)  
 
-
   def f_nn(self,pre,bd,nbd,x,layer=5):
       ''' Dimention: (nbatch,4) input = 4
                  Wi:  (4,8) 
@@ -275,7 +263,6 @@ class MPNN(ReaxFF):
       out= tf.reshape(o_,[nbd,self.batch])
       return out
 
-
   def get_tap(self,r,bd):
       if self.vdwnn:
          tp = self.f_nn('fv',bd,self.nvb[bd],[r],layer=self.vdw_layer[1])
@@ -286,7 +273,6 @@ class MPNN(ReaxFF):
               tf.math.divide(20.0,tf.pow(self.vdwcut,7.0))*tf.pow(r,7.0)
       return tp
 
-
   def fmassage(self,pre,bd,nbd,x,layer=5):
       ''' Dimention: (nbatch,4) input = 4
                  Wi:  (4,8) 
@@ -309,7 +295,6 @@ class MPNN(ReaxFF):
       out= tf.reshape(o_,[nbd,self.batch,3])
       return out
 
-
   def get_bondorder_uc(self,bd):
       self.frc[bd] = tf.where(tf.logical_or(tf.greater(self.rbd[bd],self.rc_bo[bd]),
                                             tf.less_equal(self.rbd[bd],0.001)),
@@ -331,12 +316,11 @@ class MPNN(ReaxFF):
       fpi_            = self.f_nn('fpi',bd, self.nbd[bd],[self.eterm2[bd]],layer=self.bo_layer[1])  
       fpp_            = self.f_nn('fpp',bd, self.nbd[bd],[self.eterm3[bd]],layer=self.bo_layer[1]) 
 
-      self.bop_si[bd] = fsi_*self.eterm1[bd] # consist with GULP
-      self.bop_pi[bd] = fpi_*self.eterm2[bd]
-      self.bop_pp[bd] = fpp_*self.eterm3[bd]
+      self.bop_si[bd] = fsi_ #*self.frc[bd] #*self.eterm1[bd]  
+      self.bop_pi[bd] = fpi_ #*self.frc[bd] #*self.eterm2[bd]
+      self.bop_pp[bd] = fpp_ #*self.frc[bd] #*self.eterm3[bd]
       self.bop[bd]    = tf.add(self.bop_si[bd],self.bop_pi[bd]+self.bop_pp[bd],name='BOp_'+bd)
 
-
   def get_bondorder(self,t,bd,atomi,atomj):
       Di      = tf.gather_nd(self.D[t-1],self.dilink[bd])
       Dj      = tf.gather_nd(self.D[t-1],self.djlink[bd])
@@ -367,7 +351,6 @@ class MPNN(ReaxFF):
       bo      = bosi+bopi+bopp
       return bo,bosi,bopi,bopp
 
-
   def get_bond_energy(self):
       BO = tf.zeros([1,self.batch])   # for ghost atom, the value is zero
       for bd in self.bonds:
@@ -394,7 +377,6 @@ class MPNN(ReaxFF):
           self.ebda[mol] = tf.gather_nd(EBDA,self.bdlink[mol])  
           self.ebond[mol]= tf.reduce_sum(input_tensor=self.ebda[mol],axis=0,name='bondenergy')
 
-
   def massage_passing(self):
       ''' finding the final Bond－order with a massage passing '''
       self.H.append(self.bop)                   # 
@@ -427,7 +409,6 @@ class MPNN(ReaxFF):
           Delta  = tf.reduce_sum(input_tensor=D,axis=1)
           self.D.append(Delta)                  # degree matrix
 
-
   def get_final_sate(self):     
       self.Delta  = self.D[-1]
       self.bo0    = self.H[-1]                  # fetch the final state 
@@ -462,7 +443,6 @@ class MPNN(ReaxFF):
       self.FBOT   = taper(self.BO0,rmin=self.atol,rmax=2.0*self.atol) 
       self.FHB    = taper(self.BO0,rmin=self.hbtol,rmax=2.0*self.hbtol) 
 
-
   def get_ebond(self,bd):
       Di      = tf.gather_nd(self.Delta,self.dilink[bd])
       Dj      = tf.gather_nd(self.Delta,self.djlink[bd])
@@ -497,7 +477,6 @@ class MPNN(ReaxFF):
          self.ppeng[bd]= tf.multiply(self.p['Depp_'+bd],self.bopp[bd]) 
 
          self.EBD[bd]  = - self.sieng[bd] - self.pieng[bd] - self.ppeng[bd] 
-
           
   def set_m(self):
       ''' set variable for neural networks '''
@@ -545,7 +524,6 @@ class MPNN(ReaxFF):
          self.set_wb(pref='fv',reuse_m=reuse_m,nin=1,nout=1,layer=self.vdw_layer,
                      vlist=self.bonds,nnopt=self.mpopt[-1],bias=-0.867)
 
-
   def set_wb(self,pref='f',reuse_m=True,nnopt=True,nin=8,nout=3,layer=[8,9],vlist=None,bias=0.0):
       ''' set matix varibles '''
       if self.m_ is None:
@@ -595,7 +573,6 @@ class MPNN(ReaxFF):
               self.m[pref+'wo_'+bd] = tf.Variable(tf.random.normal([layer[0],nout],stddev=0.0001), name=pref+'wo_'+bd)   
               self.m[pref+'bo_'+bd] = tf.Variable(tf.random.normal([nout], stddev=0.0001)+bias,
                                                   name=pref+'bo_'+bd)
-        
 
   def write_lib(self,libfile='ffield',loss=None):
       p_   = self.sess.run(self.p)
